//Listing 10.12 Code to register a class with the Lookup Server as a service
//XML - A Distributed Computing Paradigm - Ajay M R

//java imports.
import java.util.*;
import java.io.*;

//RMI imports.
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;

//JINI imports.
import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
import net.jini.discovery.LookupDiscovery;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceRegistration;

/**
 * This class will find the lookup services and
 * publish the proxy.<br><i>
 * NOTE: This class isn't involved in helping the proxy
 *       but it only helps it manage its JINI responsibilities. </i>
 *
 * @author :  Ravinder Khokhar
 * @version : $Revision 1.0$
 */

public class SomeService implements Runnable {

    //Leased time.
    protected final int LEASE_TIME = 10*60*1000;

    //HashMap to track new/current registrations.
    protected HashMap registrations = new HashMap();

    protected ServiceItem item;
    protected LookupDiscovery disco;

    /**
     * Inner class to listen for discovery events.
     */
    class Listener implements DiscoveryListener {

        /**
         * Invoked when we find a new lookup service.
         * @param DiscoveryEvent discovery event object which encapsulates
         *                       registrars.
         */
        public void discovered( DiscoveryEvent event ) {

            System.out.println("HelloWorldService -> Listener : Discovered "+
            "a lookup service");

            ServiceRegistrar[] newregs = event.getRegistrars();

            for ( int i=0; i<newregs.length; i++) {

                if ( !registrations.containsKey( newregs[i] ) )
                registerWithLookup( newregs[i] );

            }//for loop ends here.

        }//discovered method ends here.


        /**
         * Invoked only when we explicitly discard a lookup
         * service, not "automatically" when a lookup
         * service goes down. Once Discovered, there is NO
         * ongoing communication with a lookup service.
         * @param DiscoveryEvent discovery event object which encapsulates
         *                       registrars.
         */
        public void discarded( DiscoveryEvent event ){

            ServiceRegistrar[] deadregs = event.getRegistrars();

            for ( int i=0; i<deadregs.length; i++ )
            registrations.remove( deadregs[i] );

        }//discarded method ends here.

    }//inner class Listener ends here.


    /**
     * public Constructor.
     */
    public SomeService() throws IOException {

        // parameter description :
        // 1. null - this is the ServiceID, if you pass null
        //           a golbally unique ID is allocated.
        //
        // 2. instance of the service proxy object.
        //
        // 3. null - a list of attributes that we wish to
        //           associate with the service proxy. these
        //           attributes can be specific to any purpose
        //           like proxy for printer or scaner will have some
        //           unique attributes.
        item = new ServiceItem( null, createProxy(), null );

        //set the security manager
        if ( System.getSecurityManager() == null ) {
            System.setSecurityManager( new RMISecurityManager() );
        }

        //Search the public group, which by convention
        //is maned by an empty string
    disco = new LookupDiscovery( new String[] { "" } );

        //Install a listener.
        disco.addDiscoveryListener( new Listener() );

    }//HelloWorldService constructor ends here.

    //to create a proxy service.
    protected SomeServiceInterface createProxy() {

        return new SomeServiceProxy();

    }//SomeServiceInterface ends here.


    /**
     * This will involves remote calls, and may take while to complete.
     * Thus since it's called form discovered(). it will prevent us
     * from responding in a timely fashion to new discovery events.
     * So in future this functionality will be accomplished by spinning
     * off a new thread to do this work.
     * @param ServiceRegistrar service registrars object to register
     *                         our services.
     */
    synchronized void registerWithLookup( ServiceRegistrar registrar ) {

        ServiceRegistration registration = null;

        try {

            registration = registrar.register( item, LEASE_TIME );

        }catch( RemoteException ex) {
            System.out.println("SomeService - Coudn't register :"+
            ex.getMessage() );
            return;
        }//catch ends here.

        // if this is our first regirstration, use the service
        // ID returned to us.
        if ( item.serviceID == null ) {
            item.serviceID = registration.getServiceID();
            System.out.println("HelloWorldService: set service ID to "+
            item.serviceID);
        }//if ends here.

    }//registerWithLookup method ends here.

    /**
     *this thread does nothing but sleeps, but it
     * makes sure the VM does not exit's.
     */
    public void run(){

        while ( true ) {

            try {
                Thread.sleep(1000000);
        }catch(InterruptedException ex) {}

        }//while ends here.

    }//run ends here.


    /**
     * <i>Main entry point.</i>
     * Create's a new SomeService and start
     * background thread which will prevent VM from exiting.
     */
    public static void main(String[] args){

        try {

            // start service.
            SomeService SomeService = new SomeService();

            // assign it to a thead which will prevent
            // it from kicked out by VM
            new Thread( SomeService ).start();

        }catch(IOException ioex ){

            System.out.println("SomeService - Couldn't create service: "+
            ioex.getMessage() );

        }//catch ends here.

    }// main method ends here.

}//SomeService class ends here.

